﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Markup;
using System.Resources;
using System.IO;
using System.Reflection;
using System.Windows.Threading;
using System.Threading;

namespace SilverlightApplication16
{
  public class UserControlContentAccessor : UserControl
  {
    public static UIElement GetContent(UserControl uc)
    {
      return ((UIElement)uc.GetValue(UserControl.ContentProperty));
    }
    public static void SetContent(UserControl uc, UIElement element)
    {
      uc.SetValue(UserControl.ContentProperty, element);
    }
  }
  public static class MessageBox
  {    
    public static void ShowAsync(object content)
    {
      ShowAsync(content, null);
    }
    public static void ShowAsync(object content,
      EventHandler<MessageBoxResultEventArgs> callback)
    {
      ShowAsync(content, null, callback);
    }
    public static void ShowAsync(object content, object userState,
      EventHandler<MessageBoxResultEventArgs> callback)
    {
      ShowAsync(content, userState, callback, null);
    }
    public static void ShowAsync(object content, object userState,
      EventHandler<MessageBoxResultEventArgs> callback, Style controlTemplate)
    {
      MessageBoxControl control = new MessageBoxControl();
      control.Content = content;

      if (controlTemplate != null)
      {
        control.Style = controlTemplate;
      }
      ShowAsync(control, userState, callback);
    }
    public static void ShowAsync(MessageBoxControl control, object userState,
      EventHandler<MessageBoxResultEventArgs> callback)
    {
      UserControl uc = Application.Current.RootVisual as UserControl;      

      if (uc != null)
      {
        asyncState = userState;
        userCallback = callback;
        realVisual = UserControlContentAccessor.GetContent(uc);
        realVisual.IsHitTestVisible = false;

        parentGrid = new Grid();
        UserControlContentAccessor.SetContent(uc, parentGrid);
        parentGrid.Children.Add(realVisual);
        parentGrid.Children.Add(control);

        control.MessageBoxDismissed += OnDismissed;
      }
    }
    static void OnDismissed(object sender, MessageBoxResultEventArgs e)
    {
      MessageBoxControl control = sender as MessageBoxControl;

      UserControl uc = Application.Current.RootVisual as UserControl;

      if (uc != null)
      {
        parentGrid.Children.Clear();
        realVisual.IsHitTestVisible = true;
        UserControlContentAccessor.SetContent(uc, realVisual);
      }
      if (control != null)
      {
        control.MessageBoxDismissed -= OnDismissed;
      }
      try
      {
        if (userCallback != null)
        {
          userCallback(null, new MessageBoxResultEventArgs()
          {
            Result = e.Result,
            AsyncState = asyncState
          });
        }
      }
      finally
      {
        realVisual = null;
        parentGrid = null;
        asyncState = null;
        userCallback = null;
      }
    }    
    private static UIElement realVisual;
    private static Grid parentGrid;
    private static object asyncState;
    private static EventHandler<MessageBoxResultEventArgs> userCallback;
  }
}