﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace SilverlightApplication16
{
  public enum MessageBoxResult
  {
    Yes,
    No,
    Cancel
  }
  public class MessageBoxResultEventArgs : EventArgs
  {
    public MessageBoxResult Result { get; set; }
    public object AsyncState { get; set; }
  }
  [TemplatePart(Name = RootElement, Type = typeof(Panel))]
  [TemplatePart(Name = YesButtonElement, Type = typeof(Button))]
  [TemplatePart(Name = NoButtonElement, Type = (typeof(Button)))]
  [TemplatePart(Name = CancelButtonElement, Type = (typeof(Button)))]
  public class MessageBoxControl : ContentControl 
  {
    public event EventHandler<MessageBoxResultEventArgs> MessageBoxDismissed;

    public MessageBoxControl()
    {
      DefaultStyleKey = typeof(MessageBoxControl);
    }
    public override void OnApplyTemplate()
    {
      if (yesButton != null)
      {
        yesButton.Click -= OnYesButton;        
      }
      if (noButton != null)
      {
        noButton.Click -= OnNoButton;
      }
      if (cancelButton != null)
      {
        cancelButton.Click -= OnCancelButton;
      }
      rootElement = base.GetTemplateChild(RootElement) as Panel;
      yesButton = base.GetTemplateChild(YesButtonElement) as Button;
      noButton = base.GetTemplateChild(NoButtonElement) as Button;
      cancelButton = base.GetTemplateChild(CancelButtonElement) as Button;

      if (yesButton != null)
      {
        yesButton.Click += OnYesButton;
      }
      if (noButton != null)
      {
        noButton.Click += OnNoButton;
      }
      if (cancelButton != null)
      {
        cancelButton.Click += OnCancelButton;
      }
    }
    void OnYesButton(object sender, EventArgs args)
    {
      FireDismissed(MessageBoxResult.Yes);
    }
    void OnNoButton(object sender, EventArgs args)
    {
      FireDismissed(MessageBoxResult.No);
    }
    void OnCancelButton(object sender, EventArgs args)
    {
      FireDismissed(MessageBoxResult.Cancel);
    }
    void FireDismissed(MessageBoxResult result)
    {
      if (MessageBoxDismissed != null)
      {
        MessageBoxDismissed(this, new MessageBoxResultEventArgs() { Result = result });
      }
    }
    Button yesButton;
    Button noButton;
    Button cancelButton;    
    Panel rootElement;
    public const string RootElement = "RootElement";
    public const string YesButtonElement = "YesButtonElement";
    public const string NoButtonElement = "NoButtonElement";
    public const string CancelButtonElement = "CancelButtonElement";
  }
}